function [Y,ntpost] = NT_decode(mfcc, model, NT, TXPROB, SLOW)
% [Y,ntpost] = NT_decode(mfcc, model, NT, TXPROB, SLOW)
%    Run a viterbi decoder over a set of features to find NT
%    regions.  mfcc is matrix of features (organized into several
%    separate blocks, of lengths specified in mlen).  model is a
%    GMM structure; component NT is the NT mode.
%    TXPROB is state-to-state transition prob (0 -> default 0.0000001)
%    SLOW is a flag to use slow (but more robust) decoding.
%    Return Y as <start end 1> blocks.
%    ntpost returns a vector of NT posterior probabilities.
% 2012-04-27 Dan Ellis dpwe@ee.columbia.edu

if nargin < 3; NT = 1; end
if nargin < 4; TXPROB = 0; end
if nargin < 5; SLOW = 0; end

if TXPROB == 0
  TXPROB = 0.000000001;
end

nclust = size(model.mean,2);

% Evaluate each component gaussian over data
for i = 1:nclust
  pp(i,:) = gaussian_prob(mfcc',model.mean(:,i),squeeze(model.cov(:,:,i))); 
end

% simple transition matrix favors self-loops
tx = eye(nclust) + TXPROB;

% If more than 1 NT, allow them to loop back
for i = 1:length(NT)
  for j = 1:length(NT)
    if i ~= j
      tx(NT(i),NT(j)) = 1;
      tx(NT(j),NT(i)) = 1;
    end
  end
end

tx = tx./repmat(sum(tx,2),1,nclust);

% viterbi decode
hoptime = 0.010; % we hope...
vp = viterbi_path(ones(1,nclust)/nclust, tx, pp, SLOW);

if length(NT)>1
  NTp = max(repmat(vp,length(NT),1)==repmat(NT',1,length(vp)));
else
  NTp = (vp==NT);
end
times = hoptime*[0:(length(vp))];
%times = undo_desquelch_times(hoptime*[0:(length(vp))], DESM);

Y = [times',([NTp,0])'];

Y = lab2col_to_lab3col(Y);

% raw posterior of NT state
ntpost = pp(NT,:)./sum(pp);
