function [S,C,V] = compare_labels(T,R)
% [S,C,V] = compare_labels(T,R)
%     T and R are two sets of labels as <t_start> <t_end> <label>
%     S returns as the overall agreement (between 0 and 1); 
%     C returns a confusion matrix
%     V returns a set of merged labels (label = nlabs*T + R)
% 2012-02-28 after score_chord_idm 2008-08-24 Dan Ellis dpwe@ee.columbia.edu

nlabels = 1+max([1;T(:,3);R(:,3)]);
% Prepend 1 so empty label sets don't defeat process, min 1
% apparent label


C = zeros(nlabels, nlabels);

NOCHORD = 0;

%TT = normalize_label_stream(T);
%RR = normalize_label_stream(R);
TT = lab3col_to_lab2col(T);
RR = lab3col_to_lab2col(R);

% Now the main routine: TT and RR are two <start time> <label> 
% streams.  Calculate their total times in each pair of states
alltimes = unique(sort([TT(:,1)',RR(:,1)']));

V = [];

% track error
CC = 0;
EE = 0;
% We ignore the final symbol since we don't have an end time for it
for j = 1:length(alltimes)-1
  thistime = alltimes(j);
  nexttime = alltimes(j+1);
  segdur = nexttime - thistime;
  Tsym = lookup(TT,thistime,NOCHORD);
  Rsym = lookup(RR,thistime,NOCHORD);

  % update confusion matrix with this amount of time
  C(1+Rsym,1+Tsym) = C(1+Rsym,1+Tsym) + segdur;
  
  if Tsym == Rsym
    if Tsym ~= NOCHORD
      CC = CC + segdur;
    end
  else
    EE = EE + segdur;
  end
  V = [V;thistime, nexttime, nlabels*Tsym+Rsym];
  
end

% Actual accuracy %.  
% Exclude regions where both streams report No Chord (e.g. lead
% in/lead out)

XX = C(NOCHORD+1, NOCHORD+1);

S = (sum(diag(C))-XX) / (sum(C(:))-XX);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function L = lookup(D,T,NOCHORD)
% L = lookup(D,T,NOCHORD)
%    D is a set of rows <starttime> <label>
%    Find the interval D(i,1) <= T && D(i+1,1) > T
%    and return L = D(i,2)
%    return NOCHORD if no match
% 2008-08-24 Dan Ellis dpwe@ee.columbia.edu

% Make sure there is a zero entry, but return special symbol if it's used
% (will be superceded by a real zero entry)

D = [[0,NOCHORD];D];
%D = [D;[inf,NOCHORD]];  % never needed by design

L = D(max(find(D(:,1)<=T)),2);

