function model = gmm_est(x,modelin,its,viz)
% model = gmm_est(x,modelin,its,viz)   Estimate Gaussian mixture models
%     Model the distribution of data x with Gaussians.  
%     modelin provides initial means and variances for clusters
%       scalar <modelin> just uses that many mixture components (default 5).
%     its is the number of iterations to perform (default 20)
%     If viz is 1, plot first 2 dims at each iteration/ -1, just at end
%     model.mean(:,i) is the mean of Gaussian i
%     model.cov(:,:,i) is its covariance
%     model.weight(i) is its prior
% 2012-03-13 after gmmest.m 2001-02-11 dpwe@ee.columbia.edu
% $Header: $

if nargin < 2
  modelin = 5;
end

% Global data properties
[ndat,ndim] = size(x);
meanx = mean(x);
covx = cov(x);
% Eigen analysis of inverse covariance
%[u,s,v] = svd(inv(covx));

if isnumeric(modelin)
  nmix = modelin;
  % Random initial means around global mean, distributed like data
  %mi = (ones(nmix,1) * meanx) + (v*inv(sqrt(s))*randn(ndim, nmix))';
  model.mean = kmeansppinit(x,nmix)';
  model.cov = repmat(covx/sqrt(nmix),[1 1 nmix]);
  model.weight = ones(1,nmix)/nmix;
  model.diag = 0;
else
  model = modelin;
  nmix = size(model.mean,2);
end

if nargin < 3
  its = 20;  % was 20. then 10 for a bit, but 10 is not enough
end

if nargin < 4
  viz = 0;
end

lik = -9999;

pistuff = (2*pi) ^ -(ndim/2);

jx = zeros(ndat, nmix);

if viz ~= 0
  % precompute histogram
  if ndim == 1
    [histn, histx] = hist(x(:,1),100);
    vscale = max(histn);
  else
    minx = min(x(:,1)); maxx = max(x(:,1));
    xrange = minx:((maxx-minx)/500):maxx;
    miny = min(x(:,2)); maxy = max(x(:,2));
    yrange = miny:((maxy-miny)/500):maxy;
    histX = hist2(x(:,1:2),xrange,yrange)';
  end
end

for it = 1:its
  xlik = zeros(ndat,nmix);

  if viz == 1
    if ndim == 1
      bar(histx, histn);
      gmm_plot(model,[1],vscale*min(sqrt(squeeze(model.cov))./model.weight'),'r');
    else
      %plot(x(:,1),x(:,2),'.b');
      imagesc(xrange,yrange,log(1+histX)); axis('xy');
      gmm_plot(model,[1 2],1,'r');
    end
    pause
  end
  
  % Calculate posterior data memberships for each component
  for c = 1:nmix
    % Reconstruct covar mx
    cv = squeeze(model.cov(:,:,c));
    isnancv = max(isnan(cv(:)));
    if ~isnancv; ccv = cond(cv); end
    if isnancv || isnan(ccv) || ccv > 100000
      % gaussian is degenerating - reinitialize it
      if viz
        disp(['Warn: reinitializing component ', num2str(c), ...
              ' (cond =', num2str(ccv),')']);
      end      
      %cv
      %model.mean(:,c) = meanx;
      model.mean(:,c) = x(randi(ndat),:)';
      model.cov(:,:,c) = covx;
      cv = model.cov(:,:,c);
    end
    xmm = x' - repmat(model.mean(:,c),1,ndat);
    % Evaluate Gaussians
    if ndim == 1
      % Matlab syntax bites us
      px = exp(-0.5*   (xmm.*(inv(cv)*xmm)))/sqrt(det(cv));
    else
      px = exp(-0.5*sum(xmm.*(cv\xmm)))/sqrt(det(cv));
    end
    jx(:,c) = model.weight(c)*px';
    % Save l/hood for each pt under each mix
    xlik(:,c) = pistuff*model.weight(c)*px';
  end

  % Report data likelihood before this iteration
  olik = lik;
  lik = sum(log(sum(xlik,2)));
  if viz ~= 0 & rem(it,10) == 1
    disp(['Iteration=', num2str(it),' Log data likelihood = ', num2str(lik), ' delta=',num2str(lik-olik)]);
  end
  
  % Normalize rows of p(j|x) to be true posteriors
  jx = jx ./ (ones(nmix,1)*sum(jx'))';

  % Re-estimate model parameters one mixture component at a time
  for c = 1:nmix
    jxc = jx(:,c);
    sjxc = sum(jxc);
    model.weight(c) = sjxc/ndat;
    model.mean(:,c) = (x'*jxc)/sjxc;
    xmm = x - repmat(model.mean(:,c)',ndat,1);
    % Weighted expectation is scaled xT * x
    %cvj =  (((jxc*ones(1,ndim)) .* xmm)' * xmm) / sjxc;
    model.cov(:,:,c) =  xmm' * (repmat(jxc,1,ndim).*xmm) / sjxc;
  end

end

if viz == -1
  if ndim == 1
    bar(histx, histn);
    gmm_plot(model,[1],vscale*min(sqrt(squeeze(model.cov))./model.weight'),'r');
  else
    %plot(x(:,1),x(:,2),'.b');
    imagesc(xrange,yrange,log(1+histX)); axis('xy');
    gmm_plot(model,[1 2],1,'r');
  end    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function modelout = gmmcomp_remove(model, comps)
ncomp = size(model.mean,2);
keepix = ones(1,ncomp);
keepix(comps) = 0;
keepix = find(keepix);
modelout = model;
modelout.mean = modelout.mean(:,keepix);
modelout.cov = modelout.cov(:,:,keepix);
modelout.weight = modelout.weight(keepix);
