function R = gmm_modal_score(model, data, comps)
% R = gmm_modal_score(model, data, comps)
%   Score how "modal" each component of a GMModel is by 
%   looking at the ratio of points close to the center vs. those a 
%   little further out.
%   Calculate score only for components listed in comps (default all).
% 2012-07-05 Dan Ellis dpwe@ee.columbia.edu

if nargin < 3;
  comps = 1:size(model.mean,2);
end

% We're interested in mahalanobis distances in the low range
bins = [0:0.1:20];

% We're simply going to calculate the normalized Mahal distance of
% every point from each Gaussian in model.  Then the R score for
% each component is the ratio of the count of points within 2 sds
% of the mean, to the count between 2 and 3 sds.  

%%%% vvvvv this description is obsolete

% As a measure of separation of the points under this gauss from
% rest, we're looking for a minimum in the histogram of mahal
% distances of all points.  There will be an initial peak
% (presumably) due to the actual mode, then peaks at other
% distances due to other modes.  But if this mode is well
% separated, there should be a good minimum after the first peak.
% 
% Maybe, for every value in the histogram, calculate the max of
% values both below and above it, and take the smaller of these, so
% we're looking for a minimum in-between two large maxima.  Then
% subtract the minima from the min of the both-ways-maxes, and we
% get an index of how deep the valley is.  Choose the bottom 
% of the deepest valley by this measure, then rate it as the 
% actual value of the min as a proportion of the global highest
% peak.

MX = gmm_explode(model);
ncomp = length(MX);

for cx = 1:length(comps)
  c = comps(cx);
%  subplot(221)
%  gmm_plotdata(data, model, c);

  [gv,gp,gd] = gmm_val(MX(c),data);
  [H,V] = hist(gd,bins);

  % throw away data more than 10 sds out
  H = H(find(V<=10));
  V = V(find(V<=10));
  
  % Normalize to highest peak
  H = H./max(H);

  % % Search for deepest valley
  % bestvalleydepth = 0;
  % for i = 1:length(H);
  %   abvmax = max(H(i:end));
  %   blwmax = max(H(1:i));
  %   minmax = min(abvmax, blwmax);
  %   valleydepth = minmax - H(i);
  %   if valleydepth > bestvalleydepth
  %     bestvalleydepth = valleydepth;
  %     bestvalleypos = i;
  %     bestvalley = H(i);
  %   end
  % end
%  subplot(222)
%  plot(V,H); 
%  hold on;
%  plot([V(bestvalleypos) V(bestvalleypos)], [0 1], 'r');
%  title(sprintf('mix %d valley %.2f depth %.2f',c,bestvalley,bestvalleydepth));
%  hold off

  % We want lots of mass in 0..2 sds out, and little in 2:3 sds
%  R(cx) = sum(H(find((V<2.0))))/sum(H(find((V>2.0) & (V<3.0))));
  % How about largest value in 0..2 divided by smallest value in 1..3
  R(cx) = max(H(find(V<2.0)))/min(H(find((V>1.0) & (V<3.0))));
  %  title(sprintf('mix %d ratio %.2f',c,ratio));

%  pause

end
