function C = kmeansppinit(D,K)
% C = kmeansppinit(D,K)
%    Find initial centers using kmeans++ (Arthur & Vassilvitskii,
%    2007) method (successively in proportion to distance from
%    nearest center.
%    Data items are by row
% 2012-03-13 Dan Ellis dpwe@ee.columbia.edu

[nex,ndim] = size(D);

C = zeros(K,ndim);

% kmeans++ initialization - 
% Choose centers progressively, in proportion to squared distance
% from existing center set
C(1,:) = D(randi(nex,1),:);
for i = 2:K
  % current per-point min squared distances
  dd = min(my_sqdist(C(1:(i-1),:), D),[],1);
  % arrange as distribution from 0 to 1
  cd = [0,cumsum(dd)]/sum(dd);
  % choose in proportion to those intervals
  C(i,:) = D(max(find(rand(1)>=cd)),:);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function m = my_sqdist(p, q, A)
% SQDIST      Squared Euclidean or Mahalanobis distance.
% SQDIST(p,q)   returns m(i,j) = (p(i,:) - q(j,:))*(p(i,:) - q(j,:))'.

%  From Tom Minka's lightspeed toolbox

[pn,d] = size(p);
[qn,d] = size(q);

pmag = sum(p .* p, 2);
qmag = sum(q .* q, 2);
m = repmat(qmag', pn, 1) + repmat(pmag, 1, qn) - 2*p*q';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

