%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Y,XO] = lab2col_to_lab3col(X, XI, NOLABEL)
% [Y,XO] = lab2col_to_lab3col(X, XI, NOLABEL)
%   X is set of <t_start label> rows
%   Y returns a set of <t_start t_end label> rows
%   XI is a cell array of per-row extra information; XO is
%   corresponding output entries.
% 2012-02-28 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2;  XI = []; end
if nargin < 3;  NOLABEL = 0; end

if length(XI) == 0; XI = cell(size(X,1),1); end

XO = XI;

% sort by start time
[vv,ix] = sort(X(:,1));
X = X(ix,:);
XO = XO(ix);

if size(X,1) > 0
  % remove repeated start times, keeping last version only
  ix = find([X(1:(end-1),1) ~= X(2:end,1); 1]);

  X = X(ix,:);
  XO = XO(ix);

  % merge common labels
  keepix = [1; X(1:end-1,2) ~= X(2:end,2)];
  X = X(find(keepix),:);
  XO = XO(find(keepix));

  % convert to 3 col
  Y = [X(1:end-1,1),X(2:end,1),X(1:end-1,2)];
else
  Y = zeros(0,3);
end
  
% remove "nolabel" entries
% Do XO first, else Y will have already been modified!
XO = XO(find(Y(:,3) ~= NOLABEL));
Y = Y(find(Y(:,3) ~= NOLABEL),:);

