%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Y,XO] = lab3col_to_lab2col(X, XI, NOLABEL)
% [Y, XO] = lab3col_to_lab2col(X, XI, NOLABEL)
%   X is set of <t_start t_end label> rows
%   Y returns a set of <t_start label> rows
%   overlaps in regions in X take later label
%   gaps are assigned NOLABEL (default 0).
%   XI is a cell array of per-row extra information; XO is
%   corresponding output entries.
% 2012-02-28 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2;  XI = []; end
if nargin < 3;  NOLABEL = 0; end

nX = size(X,1);

if length(XI) == 0; XI = cell(nX,1); end

if nX == 0
  Y = zeros(0,2);
else
  Y = (sort(unique([min(min(X(:,1)),0);X(:,1);X(:,2)])));
end

XO = cell(length(Y),1);

for i = 1:length(Y)
  % which interval does this fall into?
  T = Y(i);
  II = (T >= X(:,1)) & (T < X(:,2));
  II = max(find(II));
  if length(II)
    Y(i,2) = X(II,3);
    XO(i) = XI(II);
  else
    Y(i,2) = NOLABEL;
  end
end

% Merge matching adjacent labels (by deleting successive start times)
if size(Y,1) > 1
  keepix = [1; Y(1:end-1,2) ~= Y(2:end,2)];
  Y = Y(find(keepix),:);
  XO = XO(find(keepix));
end
