function [mfcc,desm,hoptime] = nt_mfcc_desquelch(FN,DESQTH,VERBOSE)
% [m,d,t] = nt_mfccs_desquelch(FN,DESQTH,VERBOSE)
%    Return MFCCs for the soundfile named by FN
%    needed for NT detection.
%    t returns the hoptime (frame time).
%    This version applied desquelch and returns desquelch list in d.
% 2012-03-06 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2; DESQTH = -60; end
if nargin < 3; VERBOSE = 0; end

targetsr = 8000;
[X,SR] = audioread(FN,targetsr);

%% Chan F has terrible LF bonks at the start of NT - try to filter
%% them out
hpf_cf = 200;
%thr_db = -60;
[X,desm] = desquelch(X,SR, DESQTH, hpf_cf, VERBOSE);

%wavwrite(X,SR,'desquelch_debug.wav');

% keep lowest 8 dims?
dims = 1:8;

% MFCCs on 100ms windows
wintime = 0.020;
hoptime = 0.010;
mfcc = melfcc(X, SR, 'wintime', wintime ,'hoptime', hoptime, ...
              'minfreq', 100, 'maxfreq', 3500);
% Limit maxfreq to avoid being confused by clipping on NT noise
% Limit minfreq to reduce influence of squelch thumps

% plus deltas? 5 frame window?
DELTAWIN = 5;
mfcc = mfcc(dims,:);
mfcc = [mfcc;deltas(mfcc,DELTAWIN)];

% return data as rows
mfcc = mfcc';

%disp(sprintf('audio=%.3f sec, frames=%d', length(X)/SR, length(mfcc)));