function [mfcc,desm,hoptime] = nt_mfcc_vx_desquelch(FN,DESQTH,USEVX,VERBOSE)
% [m,d,t] = nt_mfccs_desquelch(FN,DESQTH,USEVX,VERBOSE)
%    Return MFCCs for the soundfile named FN
%    needed for NT detection.
%    Use DESQTH (default -60 dB) as the desquelch threshold.
%    t returns the hoptime (frame time).
%    This version applied desquelch and returns desquelch list in d.
%    This version also calculates a crude "voicing feature" as the
%    peak of the normalized autocorrelation in voice pitch range.
%    (unless USEVX is specified as 0).
% 2012-03-06 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2; DESQTH = -60; end
if nargin < 3; USEVX = 1; end
if nargin < 4; VERBOSE = 0; end

%disp(['nmvd: DESQTH=', num2str(DESQTH)]);

targetsr = 8000;
[X,SR] = audioread(FN,targetsr);

%% Chan F has terrible LF bonks at the start of NT - try to filter
%% them out
hpf_cf = 200;
%thr_db = -60;
[X,desm] = desquelch(X,SR, DESQTH, hpf_cf, VERBOSE);

%wavwrite(X,SR,'desquelch_debug.wav');

% keep lowest 8 dims?
dims = 1:8;

% MFCCs on 100ms windows
wintime = 0.020;
hoptime = 0.010;

if length(X) == 0
  % file entirely squelched
  mfcc = zeros(1,2*length(dims)+1);
else

  mfcc = melfcc(X, SR, 'wintime', wintime ,'hoptime', hoptime, ...
                'minfreq', 100, 'maxfreq', 3500);
  % Limit maxfreq to avoid being confused by clipping on NT noise
  % Limit minfreq to reduce influence of squelch thumps

  % plus deltas? 5 frame window?
  DELTAWIN = 5;
  mfcc = mfcc(dims,:);
  mfcc = [mfcc;deltas(mfcc,DELTAWIN)];

  % return data as rows
  mfcc = mfcc';

  %disp(sprintf('audio=%.3f sec, frames=%d', length(X)/SR,
  %length(mfcc)));

  if USEVX
    % Calculate voicing (normalized autocorrelation peak in pitch range)
    % This is redundant with the STFT performed in melfcc, would be
    % faster to integrate it into there.
    maxpd = 0.012;
    minpd = 0.001;
    [Z,E] = stxcorr(X,[], ...
                    round(wintime*SR), round(hoptime*SR), round(maxpd*SR));
    nxcr = size(Z,1);
    V = max(Z((round(nxcr/2)+round(minpd*SR)):end,:))./E;
    mfcc = [mfcc,V'];
  end
end
